package Sample::Module;

sub foo {
 	# foo foo foo
}

sub bar {
	# bar bar bar
}

1;

__END__

=head1 NAME

Sample::Module - Sample Module

=head1 SYNOPSIS

  blah blah blah

=head1 MODULES

=head2 $foo = foo(@_)

foo foo foo

=head2 bar

bar bar bar

=head1 SEE ALSO

L<Sample::Project>

